#ifndef CASSETTE_H_
#define CASSETTE_H_

#include <stdio.h>		/* for FILE and FILENAME_MAX */

#include "config.h"
#include "atari.h"		/* for UBYTE */

#define CASSETTE_DESCRIPTION_MAX 256

extern char CASSETTE_filename[FILENAME_MAX];
extern char CASSETTE_description[CASSETTE_DESCRIPTION_MAX];
typedef enum {
	CASSETTE_NONE,
	CASSETTE_READ_ONLY,
	CASSETTE_WRITE_ONLY,
	CASSETTE_READ_WRITE
} CASSETTE_status_t;
extern CASSETTE_status_t CASSETTE_status;

typedef enum {
	CASSETTE_OP_NONE,
	CASSETTE_OP_READ,
	CASSETTE_OP_WRITE
} CASSETTE_op_t;
extern CASSETTE_op_t CASSETTE_operation;

/* Used in Atari800_Initialise during emulator initialisation */
int CASSETTE_Initialise(int *argc, char *argv[]);
void CASSETTE_Exit(void);
/* Config file read/write */
int CASSETTE_ReadConfig(char *string, char *ptr);
void CASSETTE_WriteConfig(FILE *fp);

int CASSETTE_Insert(void);
void CASSETTE_Remove(void);
int CASSETTE_CreateCAS(char const *filename, char *description);
int CASSETTE_CreateWAV(char const *filename);
int CASSETTE_CreateRaw(char const *filename);

/* Returns TRUE if the file is a cassette image. */
int CASSETTE_FileSupported(char const *filename);

extern int CASSETTE_hold_start;
extern int CASSETTE_hold_start_on_reboot; /* preserve hold_start after reboot */
extern int CASSETTE_press_space;

extern int CASSETTE_loaded; /* Is cassette loaded? */
extern int CASSETTE_record_pressed; /* Is cassette record button pushed? */

unsigned int CASSETTE_GetSize(void);
unsigned int CASSETTE_GetPosition(void); /* Counted from 1 */
void CASSETTE_Seek(unsigned int position);
/* If false, then current file's position is given in seconds. Value is
   undefined when a tape is not inserted. */
extern int CASSETTE_position_in_blocks;

#ifdef SYNCHRONIZED_SOUND
/* Audio output functions */
extern int CASSETTE_play_audio;
extern unsigned int CASSETTE_audio_volume;
enum {
	CASSETTE_DEFAULT_AUDIO_VOLUME = 50,
	CASSETTE_MAX_AUDIO_VOLUME = 99
};

void CASSETTE_InitAudio(unsigned int samplerate, unsigned int n_pokeys, int b16);
void CASSETTE_UpdateSound(void *sndbuffer, int sndn);
#endif /* SYNCHRONIZED_SOUND */

/* Functions used by unpatched SIO */
/* Loading */
int CASSETTE_IOLineStatus(void);
void CASSETTE_TapeMotor(int onoff);
void CASSETTE_AddScanLine(void);

/* Saving */
void CASSETTE_ToggleRecord(void);

/* Support for turbo systems */
enum CASSETTE_TURBO_TYPE_t {
	CASSETTE_TURBO_NONE = 0,
	CASSETTE_TURBO_AST_XC12,
	CASSETTE_TURBO_BLIZZARD,
	CASSETTE_TURBO_MANUAL,
	CASSETTE_TURBO_MANUAL_REV
};
extern enum CASSETTE_TURBO_TYPE_t CASSETTE_turbo_type;

void CASSETTE_SetTurboType(enum CASSETTE_TURBO_TYPE_t type);
extern int CASSETTE_turbo_state;
void CASSETTE_ToggleTurboState(void);
void CASSETTE_SetTurboTolerance(int value);
extern int CASSETTE_turbo_tolerance;
enum {
	CASSETTE_DEFAULT_TURBO_TOLERANCE = 3,
	CASSETTE_MAX_TURBO_TOLERANCE = 99
};

void CASSETTE_UpdateSioDataOut(int value);

/* Functions used by patched SIO */
/* SIO_Handler() */
int CASSETTE_AddGap(int gaptime);
int CASSETTE_ReadWithPatch(int length, UWORD dest_addr);
int CASSETTE_WriteWithPatch(int length, UWORD src_addr);
extern UBYTE CASSETTE_buffer[65536];
/* Other */
void CASSETTE_LeaderLoad(void);
void CASSETTE_LeaderSave(void);

#endif /* CASSETTE_H_ */
